
@import <AppKit/AppKit.j>
@import "TargetView.j"
@import "TargetObject.j"

var kAnimationDelay = 0.03333333;

@implementation TargetGallery : CPView 
{
    BOOL _isEditing @accessors(property=isEditing);
    CPMutableArray _targets @accessors(readonly, property=targets);
    int _animationID;
}

- (id)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        _targets = [];
    }
    return self;
}

- (void)mouseDown:(CPEvent) event {
    // In edit mode, let super route the event as normal 
    if ([self isEditing]) 
        return [super mouseDown:event];
}

- (CPView)hitTest:(CGPoint)aPoint {
    if ([self isEditing]) {
        return self;
    }
    
    // Not in editing mode, let super (NSView) do its normal hit testing.
    return [super hitTest:aPoint];
}

- (void)addTarget:(TargetObject)target {
    [_targets addObject:target];
    [self addSubview:[target view]];
}

- (TargetObject)targetForView:(CPView)view {
    var targetEnumerator = [_targets objectEnumerator],
        target = nil;
    while ((target = [targetEnumerator nextObject]) !== nil) {console.log(": "+[target view]+" -- "+view);
        if ([target view] === view) 
            return target;
    }
    
    return nil;
}

- (TargetObject)targetAtPoint:(CGPoint)point {
    return [self targetForView:[super hitTest:point]];
}


- (void)startAnimating {
    // cancel any outstanding animation request before requesting a new one.
    [self stopAnimating];
    _animationID = setInterval(function(){[self moveTargets:nil];}, 33);
}

- (void)stopAnimating {
    //[[self class] cancelPreviousPerformRequestsWithTarget:self];
    if (_animationID !== nil)
        clearInterval(_animationID);
    _animationID = nil;
}


- (void)moveTargets:(id)ignored {
    // Adjust each target by its velocity
    var targetEnumerator = [_targets objectEnumerator],
        target = nil;
    while ((target = [targetEnumerator nextObject]) !== nil) {
        var velocity = [target velocity];

        if (!CGPointEqualToPoint(CGPointMakeZero(), velocity)) {
            var frame = [[target view] frame];
            frame.origin.x += velocity.x * kAnimationDelay;
            frame.origin.y += velocity.y * kAnimationDelay;
            
            if (!CGRectIntersectsRect(frame, [self bounds])) {
                // Take horizontal wrap around into consideration.
                if (velocity.x < 0) {
                    if (CGRectGetMaxX(frame) < CGRectGetMinX([self bounds])) {
                        frame.origin.x = CGRectGetMaxX([self bounds]) - 1;
                    }
                } else if (velocity.x > 0) {
                    if (CGRectGetMinX(frame) > CGRectGetMaxX([self bounds])) {
                        frame.origin.x = CGRectGetMinX([self bounds]) - CGRectGetWidth(frame) + 1;
                    }
                }
                
                // Take vertical wrap around into consideration.
                if (velocity.y < 0) {
                    if (CGRectGetMaxY(frame) < CGRectGetMinY([self bounds])) {
                        frame.origin.y = CGRectGetMaxY([self bounds]) - 1;
                    }
                } else if (velocity.y > 0) {
                    if (CGRectGetMinY(frame) > CGRectGetMaxY([self bounds])) {
                        frame.origin.y = CGRectGetMinY([self bounds]) - CGRectGetHeight(frame) + 1;
                    }
                }
            }
            
            
            [[target view] setFrameOrigin:frame.origin];
        }
    }
}


@end
