@import <AppKit/AppKit.j>
@import "TargetObject.j"
@import "TargetGallery.j"

var kTargetSize = 64.0;

@implementation TargetGalleryController : CPWindowController {
    @outlet TargetGallery _targetGallery;
    TargetObject _dragTarget;
    CPTimeInterval _lastDragTime;
    CPPoint _lastDragPoint;
    CPPoint _dragVelocity;
}

- (void)awakeFromCib {
    [_targetGallery setIsEditing:NO];
    
    // Create a bunch of targets all lined up and start animating them.
    var target;
    var targetFrame = CGRectMake(CGRectGetMaxX([_targetGallery bounds]) - kTargetSize / 2.0, CGRectGetMidY([_targetGallery bounds]), kTargetSize, kTargetSize);
    
    do {
        target = [[TargetObject alloc] init];
        [target setView:[[TargetView alloc] initWithFrame:targetFrame]];
        [target setVelocity:CGPointMake(50.0, 0.0)];
        [_targetGallery addTarget:target];
        targetFrame.origin.x -= CGRectGetWidth(targetFrame) + kTargetSize / 2.0;
    }while (targetFrame.origin.x > 0);
    
    [_targetGallery startAnimating];
}

- (void)mouseDown:(CPEvent)event {
    var targetLocation = [_targetGallery convertPoint:[event locationInWindow] fromView:nil];
    _dragTarget = [_targetGallery targetAtPoint:targetLocation];
    
    if(!_dragTarget) return;
    
    _lastDragTime = [event timestamp];
    _lastDragPoint = targetLocation;
    _dragVelocity = CGPointMakeZero();
    [_dragTarget setVelocity:CGPointMakeZero()];

    [self mouseDragged:event];
}

- (void)mouseDragged:(CPEvent)event {
    if(!_dragTarget) return;
    
    var targetLocation = [_targetGallery convertPoint:[event locationInWindow] fromView:nil];
    var timeDelta = ([event timestamp] - _lastDragTime) / 1000;

    if(timeDelta > 0) {
        _dragVelocity.x = (targetLocation.x - _lastDragPoint.x) / timeDelta;
        _dragVelocity.y = (targetLocation.y - _lastDragPoint.y) / timeDelta;
    }
    _lastDragTime = [event timestamp];
    _lastDragPoint = targetLocation;
    
    var targetFrame = [[_dragTarget view] frame];
    targetFrame.origin.x = targetLocation.x - CGRectGetWidth(targetFrame) / 2.0;
    targetFrame.origin.y = targetLocation.y - CGRectGetHeight(targetFrame) / 2.0;
    [[_dragTarget view] setFrameOrigin:targetFrame.origin];
}

- (void)mouseUp:(CPEvent)event {
    if (!_dragTarget) return;

    [_dragTarget setVelocity:_dragVelocity];
    _dragTarget = nil;
}

- (IBAction)toggleGalleryEditing:(id)sender {
    var isEditing = [_targetGallery isEditing];
    
    // Update the text of the toolbar item since it is a toggle.
    if (isEditing) {
        [sender setLabel:@"Edit"];
    } else {
        [sender setLabel:@"Shoot"];
    }
    
    // Flip the state.
    [_targetGallery setIsEditing:!isEditing];
}

-(@action)stopTargets:(id)sender {
    // Don't stop animating, just set all the targets' velocity to 0 so they don't move.
    var targetEnumerator = [[_targetGallery targets] objectEnumerator],
        target = nil;
    while ((target = [targetEnumerator nextObject]) !== nil) {
        [target setVelocity:CGPointMakeZero()];
    }
}

@end
