@import <AppKit/AppKit.j>

@implementation TargetView : CPView
{
    CPImageView _imageView;
    TargetDrawView _targetDrawView;
}

- (id)initWithFrame:(CGRect)frame 
{
    self = [super initWithFrame:frame];
    if (self) {
        _imageView = [[CPImageView alloc] initWithFrame:[self bounds]];
        [self addSubview:_imageView];
        
        [_imageView setImage:[[CPImage alloc] initWithContentsOfFile:[[CPBundle mainBundle] pathForResource:"bullseye.png"]]];
        
        _targetDrawView = [[TargetDrawView alloc] initWithFrame:[self bounds]];

        [_targetDrawView setHitTests:NO];
        [_imageView setHitTests:NO];

        [self addSubview:_targetDrawView];
    }
    return self;
}

- (void)mouseDown:(CPEvent)event 
{
    // We use an NSBezierPath to store and draw the clicked points. We create it lazily on the first click in the view.
    if (!_targetDrawView._path){
        _targetDrawView._path = [CPBezierPath bezierPath];
    }
    
    // Convert the clicked point into the views coordinate space and add the green circle to the bezierPath. Any point within the bounds of the view is considered a hit. An interesting improvement would be to make sure the clicked point occurred on an opaque portion of the image.
    var cursorLocation = [event locationInWindow];
    cursorLocation = [self convertPoint:cursorLocation fromView:nil];
    
    var kHoleRadius = 6.0;
    var kHoleDiameter = kHoleRadius * 2.0;
    
    var holeFrame = CPMakeRect(cursorLocation.x - kHoleRadius, cursorLocation.y - kHoleRadius, kHoleDiameter, kHoleDiameter);
    [_targetDrawView._path appendBezierPath:[CPBezierPath bezierPathWithOvalInRect:holeFrame]];
    
    [_targetDrawView setNeedsDisplayInRect:holeFrame];
}

@end

@implementation TargetDrawView : CPView
{
    CPBezierPath _path;
}

- (void)drawRect:(CGRect)dirtyRect
{    
    // All we need to draw are the green circles where the user clicked.
    [[CPColor greenColor] set];
    [_path fill];
}

@end
