/*
     File: TargetGalleryController.m
 */

#import "TargetGalleryController.h"
#import "TargetGallery.h"
#import "TargetObject.h"
#import "TargetView.h"


static const CGFloat kTargetSize = 64.0f;

@implementation TargetGalleryController

- (void)awakeFromNib {
    [_targetGallery setIsEditing:NO];
    
    // Create a bunch of targets all lined up and start animating them.
    TargetObject *target;
    NSRect targetFrame = NSMakeRect(NSMaxX(_targetGallery.bounds) - kTargetSize / 2.0f, NSMidY(_targetGallery.bounds), kTargetSize, kTargetSize);
    
    do {
        target = [[TargetObject alloc] init];
        target.view = [[[TargetView alloc] initWithFrame:targetFrame] autorelease];
        target.velocity = NSMakePoint(50.0f, 0.0f);
        [_targetGallery addTarget:target];
        [target release];
        targetFrame.origin.x -= NSWidth(targetFrame) + kTargetSize / 2.0f;
    }while (targetFrame.origin.x > 0);
    
    [_targetGallery startAnimating];
}


#pragma mark NSResponder

// It is reasonable to have the TargetGalleryView handle the dragging of the targets in edit mode. However, I do it here to demonstrate how the mouse events flow up the responder chain to the window controller.
- (void)mouseDown:(NSEvent *)event {
    NSPoint targetLocation = [_targetGallery convertPoint:event.locationInWindow fromView:nil];
    _dragTarget = [_targetGallery targetAtPoint:targetLocation];
    
    if(!_dragTarget) return;
    
    _lastDragTime = event.timestamp;
    _lastDragPoint = targetLocation;
    _dragVelocity = NSZeroPoint;
    _dragTarget.velocity = NSZeroPoint;

    [self mouseDragged:event];
}

- (void)mouseDragged:(NSEvent *)event {
    if(!_dragTarget) return;
    
    NSPoint targetLocation = [_targetGallery convertPoint:event.locationInWindow fromView:nil];
    NSTimeInterval timeDelta = event.timestamp - _lastDragTime;
    
    if(timeDelta > 0) {
        _dragVelocity.x = (targetLocation.x - _lastDragPoint.x) / timeDelta;
        _dragVelocity.y = (targetLocation.y - _lastDragPoint.y) / timeDelta;
    }
    _lastDragTime = event.timestamp;
    _lastDragPoint = targetLocation;
    
    NSRect targetFrame = _dragTarget.view.frame;
    targetFrame.origin.x = targetLocation.x - NSWidth(targetFrame) / 2.0f;
    targetFrame.origin.y = targetLocation.y - NSHeight(targetFrame) / 2.0f;
    [_dragTarget.view setFrameOrigin:targetFrame.origin];
}

- (void)mouseUp:(NSEvent *)event {
    if (!_dragTarget) return;

    _dragTarget.velocity = _dragVelocity;
    _dragTarget = nil;
}


#pragma mark API

- (IBAction)toggleGalleryEditing:(id)sender {
    BOOL isEditing = [_targetGallery isEditing];
    
    // Update the text of the toolbar item since it is a toggle.
    if (isEditing) {
        [sender setLabel:NSLocalizedString(@"Edit", "Switch Target Gallery to edit mode")];
    } else {
        [sender setLabel:NSLocalizedString(@"Shoot", "Switch Target Gallery to shooting mode")];
    }
    
    // Flip the state.
    [_targetGallery setIsEditing:!isEditing];
}

-(IBAction)stopTargets:(id)sender {
    // Don't stop animating, just set all the targets' velocity to 0 so they don't move.
    for (TargetObject *object in [_targetGallery targets]) {
        object.velocity = NSZeroPoint;
    }
}

@end
