/*
     File: TargetView.m 
 */

#import "TargetView.h"

@implementation TargetView

- (id)initWithFrame:(NSRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        self.image = [NSImage imageNamed:@"bullseye"];
    }
    return self;
}

- (void)dealloc {
    [_path release];
    [super dealloc];
}


#pragma mark NSResponder

- (void)mouseDown:(NSEvent *)event {
    // We use an NSBezierPath to store and draw the clicked points. We create it lazily on the first click in the view.
    if (!_path){
        _path = [[NSBezierPath bezierPath] retain];
    }
    
    // Convert the clicked point into the views coordinate space and add the green circle to the bezierPath. Any point within the bounds of the view is considered a hit. An interesting improvement would be to make sure the clicked point occurred on an opaque portion of the image.
    NSPoint cursorLocation = [event locationInWindow];
    cursorLocation = [self convertPoint:cursorLocation fromView:nil];
    
    static const CGFloat kHoleRadius = 6.0f;
    CGFloat kHoleDiameter = kHoleRadius * 2.0f;
    
    NSRect holeFrame = NSMakeRect(cursorLocation.x - kHoleRadius, cursorLocation.y - kHoleRadius, kHoleDiameter, kHoleDiameter);
    [_path appendBezierPath:[NSBezierPath bezierPathWithOvalInRect:holeFrame]];
    
    [self setNeedsDisplayInRect:holeFrame];
}


#pragma mark NSView

- (void)drawRect:(NSRect)dirtyRect {
    // Let super handle drawing of the image.
    [super drawRect:dirtyRect];
    
    // All we need to draw are the green circles where the user clicked.
    [[NSColor greenColor] set];
    [_path fill];
}

@end
